@@ -20,7 +20,6 @@ MAX_PENDING_SUBJECTS = 50
 MAX_UPLOAD_FILE_SIZE = 1024 * 1024
 CURRENT_STATE_VERSION = 1
 
-
 @cli.group(name='subject-set')
 def subject_set():
     """Contains commands for managing subject sets."""
@@ -304,6 +303,10 @@ def upload_subjects(
             return False
         return True
 
+    def get_index_fields(headers):
+        index_fields = [header.lstrip('%') for header in headers if header.startswith('%')]
+        return ",".join(str(field) for field in index_fields)
+
     subject_set = SubjectSet.find(upload_state['subject_set_id'])
     if not resumed_upload:
         subject_rows = []
@@ -312,8 +315,15 @@ def upload_subjects(
                 file_root = os.path.dirname(manifest_file)
                 r = csv.reader(manifest_f, skipinitialspace=True)
                 headers = next(r)
+                # update set metadata for indexed sets
+                index_fields = get_index_fields(headers)
+                if index_fields:
+                    subject_set.metadata['indexFields'] = index_fields
+                    subject_set.save()
+                # remove leading % from subject metadata headings
+                cleaned_headers = [header.lstrip('%') for header in headers]
                 for row in r:
-                    metadata = dict(zip(headers, row))
+                    metadata = dict(zip(cleaned_headers, row))
                     files = []
                     if not upload_state['file_column']:
                         upload_state['file_column'] = []
@@ -323,18 +333,20 @@ def upload_subjects(
                                 upload_state['file_column'].append(
                                     field_number,
                                 )
-                                if not validate_file(file_path):
+                                if validate_file(file_path):
+                                    files.append(file_path)
+                                elif not upload_state['allow_missing']:
                                     return -1
-                                files.append(file_path)
                     else:
                         for field_number in upload_state['file_column']:
                             file_path = os.path.join(
                                 file_root,
                                 row[field_number - 1]
                             )
-                            if not validate_file(file_path):
+                            if validate_file(file_path):
+                                files.append(file_path)
+                            elif not upload_state['allow_missing']:
                                 return -1
-                            files.append(file_path)
 
                     for field_number, _mime_type in zip(
                         upload_state['remote_location'],
@@ -553,6 +565,59 @@ def delete(force, subject_set_ids):
         subject_set.delete()
 
 
+@subject_set.command(name="download-classifications")
+@click.argument('subject-set-id', required=True, type=int)
+@click.argument('output-file', required=True, type=click.File('wb'))
+@click.option(
+    '--generate',
+    '-g',
+    help="Generates a new export before downloading.",
+    is_flag=True
+)
+@click.option(
+    '--generate-timeout',
+    '-T',
+    help=(
+        "Time in seconds to wait for new export to be ready. Defaults to "
+        "unlimited. Has no effect unless --generate is given."
+    ),
+    required=False,
+    type=int,
+)
+def download_classifications(
+    subject_set_id,
+    output_file,
+    generate,
+    generate_timeout
+):
+    """
+    Downloads a subject-set specific classifications export for the given subject set.
+
+    OUTPUT_FILE will be overwritten if it already exists. Set OUTPUT_FILE to -
+    to output to stdout.
+    """
+
+    subject_set = SubjectSet.find(subject_set_id)
+
+    if generate:
+        click.echo("Generating new export...", err=True)
+
+    export = subject_set.get_export(
+        'classifications',
+        generate=generate,
+        wait_timeout=generate_timeout
+    )
+
+    with click.progressbar(
+        export.iter_content(chunk_size=1024),
+        label='Downloading',
+        length=(int(export.headers.get('content-length')) / 1024 + 1),
+        file=click.get_text_stream('stderr'),
+    ) as chunks:
+        for chunk in chunks:
+            output_file.write(chunk)
+
+
 def echo_subject_set(subject_set):
     click.echo(
         u'{} {}'.format(
