@@ -3,6 +3,8 @@ from panoptes_cli.commands.project import ls
 
 import unittest
 
+import warnings
+import sys
 
 class TestProject(unittest.TestCase):
     def __init__(self, *args, **kwargs):
@@ -10,6 +12,13 @@ class TestProject(unittest.TestCase):
         self.runner = CliRunner()
 
     def test_ls_id_public(self):
+        if sys.version_info >= (3, 0):
+            # avoid client socket warnings polluting our test results
+            # this can go when the underlying client issue is fixed
+            # https://github.com/zooniverse/panoptes-python-client/issues/270
+            warnings.filterwarnings(
+                action="ignore", message="unclosed", category=ResourceWarning)
+
         result = self.runner.invoke(ls, ['--project-id', '1'])
         self.assertEqual(
             result.output,
